#!/usr/bin/env ruby

if( ARGV.include?("--help") )
  print <<-EOF
  --help              print this messages
  --enable-OPTION     enable yarv option
  --disable-OPTION    disable yarv option
  --disable-opts      disable all optimization
  --disable-opt-unifs disable unification optimization
  
OPTION:
  
  optimization option (default enable):
  * opt-direct-threaded-code
  * opt-basic-operations
  * opt-operands-unification
  * opt-instructions-unification
  * opt-stack-caching
  * opt-inline-method-cache
  * opt-unify-all-combination

  others (default disable):
  * opt-token-threaded-code
  * opt-call-threaded-code
  * collect-usage-analysis
  * opt-jit-compile          (experimental, support only few insns)
  * test-aot-compile         (experimental)
  * fake-inline-method-cache (dengerous)
  * support-goto             (only joke)
  
  EOF
  exit(0)
end

if RUBY_VERSION != '1.9.0'
  raise "YARV requires ruby 1.9.0"
end

unless defined?(::YARV_PATCHED)
  raise "ruby should be applied yarv patch"
end

require 'mkmf.rb'

# it's for debug
CONFIG['LDSHARED'].sub!(/-s\b/, '')
$opt_defs = []

##
def check_arg arg, default=nil
  if enable_config(arg, default)
    $opt_defs.push "-D#{arg.upcase.tr_s("^A-Z0-9_", "_")}"
  end
end

default_flag = true
if arg_config('--disable-opts')
  default_flag = false
end


# optimizer
check_arg('opt-direct-threaded-code', default_flag)
check_arg('opt-token-threaded-code', false)
check_arg('opt-call-threaded-code', false)

check_arg('opt-basic-operations', default_flag)
check_arg('opt-operands-unification', default_flag)
check_arg('opt-instructions-unification', default_flag)
check_arg('opt-inline-method-cache', default_flag)
check_arg('opt-stack-caching'   , default_flag)
check_arg('opt-unify-all-combination', default_flag)

check_arg('opt-jit-compile', false)

check_arg('collect-usage-analysis', false)

# danger
check_arg('test-aot-compile', false)
check_arg('fake-inline-method-cache', false)
check_arg('support-goto', false)

if arg_config('--disable-opt-unifs')
  $opt_defs.delete_if{|e|
    /UNIFICATION/ =~ e
  }
end

have_type("ulong", "sys/types.h")

$cleanfiles += %w( *.inc ) - %w(vm_evalbody.inc call_cfunc.inc)

$defs.concat $opt_defs

puts
puts "YARV options:"
puts $opt_defs.map{|e| "\t" + e}
puts
create_makefile('yarvcore')

if macro_defined?("__GNUC__", "") && try_compile(<<-EOS, '-fno-crossjumping')
  int main(){
    return 0;
  }
EOS
open("Makefile", "a"){|f|
f.puts <<EOS

vm.o : CFLAGS += -fno-crossjumping

# for GCC
vm.asm:
	$(CC) $(CFLAGS) -fno-reorder-blocks -fno-crossjumping $(CPPFLAGS) -dpA -S -o vm.asm.s $(srcdir)/vm.c
	$(RUBY) $(srcdir)/rb/asm_parse.rb vm.asm.s > vm.asm.each

vm.cpp:
	$(CPP) $(CPPFLAGS) $(CFLAGS) $(srcdir)/vm.c > vm.cpp.txt
EOS
  }
end
